var express = require('express');
var router = express.Router();
var readLink = require('../read-link');
var Account = require('../models/account');
var Link = require('../models/link');

router.use(function(req, res, next) {
  if(!req.user) {
    res.redirect('/auth/twitter');
  }

  Link.find()
  .where('_creator').equals(req.user._id)
  .sort('-addedDate')
  .exec(function(err, links, count) {
    if(err) {
      res.status(400);
      res.render('error', {
        message: 'Could not load links',
        error: err
      });
      return;
    }
    res.links = links;
    next();
  })
})

router.get('/', function(req, res) {

  res.render('links', {
    links: res.links,
    title: 'Your links',
    linkid: null,
    user: req.user ? req.user : null
  })
})

router.post('/', function(req, res) {

  readLink(req.param('link'), function(err, summary) {
    if(err) {
      res.status(400);
      res.send('Failed to fetch link: '+req.param('link'));
      return;
    }

    new Link({
      url: req.param('link'),
      _creator: req.user._id,
      title: summary.title,
      image: summary.image,
      sentiment: summary.sentiment,
      keywords: summary.topics,
      words: summary.words,
      difficulty: summary.difficulty,
      readingTime: summary.minutes
    }).save(function(err, link, count) {
      if(err) {
        res.status(400);
        res.render('error', {
          message: 'Could not save link',
          error: err
        })
        return
      }

      res.redirect('/links/'+link._id);
    })
  });
})

router.route('/:linkid')
  .all(function(req, res, next) {
    res.linkid = req.params.linkid;

    Link.findOne({
      '_creator': req.user._id, 
      '_id': res.linkid
    }, function(err, link, count) {
      if(err || !link) {
        res.status(404);
        res.render('error', {message: 'Not found'});
        return;
      }

      res.link = link;
      next();
    })
  })
  .get(function(req, res) {
    res.render('links', {
      links: res.links,
      title: 'Your links',
      linkid: res.linkid,
      link: res.link,
      user: req.user ? req.user : null
    })
  })
  .put(function(req, res) {
    readLink(res.link.url, function(err, summary) {
      if(err) {
        res.status(400);
        res.send('Failed to fetch link: '+req.param('link'));
        return;
      }

      res.link.addedDate = Date.now();
      res.link.title = summary.title,
      res.link.image = summary.image,
      res.link.sentiment = summary.sentiment,
      res.link.keywords = summary.topics,
      res.link.words = summary.words,
      res.link.difficulty = summary.difficulty,
      res.link.readingTime = summary.minutes

      res.link.save(function(err, link) {
        if(err) {
          res.status(400);
          res.send('Could not save link')
        }

        res.send('OK');
      })
    })
  })
  .delete(function(req, res) {
    res.link.remove(function(err, link) {
      res.send('Remove successful');
    })
  })

module.exports = router;